
#include "pw/core/debug.hpp"
#include "pw/visual/pipeline.hpp"
#include "pw/visual/shader.hpp"
#include "pw/visual/framebuffer.hpp"
#include "pw/visual/texture.hpp"

#include "pw/core/size.hpp"


#include "runtime_lua.hpp"

namespace pw {

void register_visual_function(sol::state&,sol::table &ns)
{

    ns.new_usertype<pipeline>("pipeline"
							  ,"create",&pipeline::create
                              ,"draw",&pipeline::draw
                              );

    ns.new_usertype<shader>("shader"
                            ,sol::constructors<shader()>()
                            ,"ready",sol::readonly_property(&shader::ready)
                            ,"use",&shader::use
                            ,"build",&shader::build
                            ,"source",&shader::source
                            ,"set_source",&shader::set_source
                            ,"set_uniforms",&shader::set_uniforms
                            ,"set_uniform_float",&shader::set_uniform<float>
                            ,"set_uniform_mat4",&shader::set_uniform<matrix4x4&>
                            ,"set_uniform_vec4",&shader::set_uniform<vector4&>
                            ).new_enum<false>("shader_type"
                                       ,"fragment",shader::code_type::fragment
                                              ,"vertex",shader::code_type::vertex
                                              ,"geometry",shader::code_type::geometry
                                              ,"compute",shader::code_type::compute);


    ns.new_usertype<render_pass>("render_pass"
                                ,"submit",&render_pass::submit
                                 );


    ns.new_usertype<renderer>("renderer"
                                   ,sol::constructors<renderer(),renderer(const geometry&)>()
                              ,"create",&renderer::create
                              ,"ready",sol::readonly_property(&renderer::ready)
                              ,"release",&renderer::release
                              ,"draw",&renderer::draw
                              );

    ns.new_usertype<framebuffer>("framebuffer"
                                 ,sol::constructors<framebuffer()>()
                                 ,"create",&framebuffer::create
                                 ,"bind",&framebuffer::bind
                                 ,"unbind",&framebuffer::unbind
                                 ,"blit",&framebuffer::blit);

    ns.new_usertype<texture>("texture"
                             ,sol::constructors<texture(),texture(texture::image_ref,texture::data_layout)>()
                             ,"image",sol::property(&texture::set_image,&texture::image)
                             );
}

PW_REGISTER_LUA(visual)

}
